% DFS search for random csp problem, P is a problem instance
% where P(i,j,k) is 1 if the k'th pair can have an assignment (i,j)
% return: solvable is 1 if problem is solvable (extracting the solution is
% trivial)
function solvable = dfs(P)

n = round((1 + sqrt(1 + 8*size(P,3)))/2);
m = size(P,1);

solvable = dfs_rec(1:n, ones([m n]), n, P);

function solvable = dfs_rec(unassigned, state, n, P)

    L = length(unassigned);
    
    % get number of constraints
    constraints = sum(state(:,unassigned), 1);
    [y,I] = sort(constraints); % ascending order
    unassigned = unassigned(I);
    
    % sort by constraints
    solvable = 0;
    
    % there is an unassigned variable with no more possible values
    if( constraints(1) == 0 )
        return;
    end
    
    % test for completion
    if( L == 1 )
        if( constraints(1) > 0 )
            solvable = 1;
        end
        return;
    end
            
    for var_index = 1:L
        var = unassigned(var_index); % get unassigned variable    
        possible = find(state(:,var)); % possible values
    
        if( isempty(possible) )
            % backtrack
            solvable = 0;
        end
    
        newunassigned = unassigned(find(1:L ~= var_index));
        
        for p = possible'
            % propagate constraints
            newstate = state;
            
            for i = newunassigned
                C = getconstraints(var,i,n,P);
                newstate(:,i) = newstate(:,i) .* C(:,p);
            end

            if( dfs_rec(newunassigned, newstate, n, P) )
                solvable = 1;
                return;
            end
        end
        
    end

