#!/usr/bin/perl

#4 phases (stages 0, 2, 4, 6)
#3 transitions (stages 1, 3, 5)
#5 state variables
#	0	KOP
#	1	PG
#	2	SI
#	3	SS
#	4	PS
#State variable CNF index is [stage]/2*5 + [state variable index] + 1
#14 actions
#	0	BPG
#	1	BSI
#	2	BSS
#	3	BPS
#	4	MNKOP
#	5	MKOP
#	6	MNPG
#	7	MPG
#	8	MNSI
#	9	MSI
#	10	MNSS
#	11	MSS
#	12	MNPS
#	13	MPS
#Action CNF index is 20 + ([transition]-1)/2*14 + [action index] + 1

#Total CNF variables is 5*4 + 14*3 = 62

#Start constraint:
#	5 clauses (one for each state variable)
#Goal constraint:
#	1 clause (just the have plan solution variable)
#Action constraints:
#	Each transition needs:
#		19 for precondition requirements
#			2	BPG
#			3	BSI
#			1	BSS
#			3	BPS
#			1	M___ (10 of these)
#		14 for postcondition assertions
#		10 for each literal requiring an action to reach it
#			5	only M for NOT each state variable
#			5	M or build for each state variable
#		= 43 constraints
#	=> 3*43 = 129 constraints total

print "p cnf 62 135\n";

#Start
print "1 0\n";
print "-2 0\n";
print "-3 0\n";
print "-4 0\n";
print "-5 0\n";

#Goal
print "20 0\n";

#Transitions
for $i qw(1 3 5) {
  $basevar = 20+($i-1)/2*14 + 1;
  $prevstagebase = ($i-1)/2*5 + 1;
  $nextstagebase = ($i+1)/2*5 + 1;

  #BPG
  #preconditions: KOP and !PG
  print "" . (-($basevar+0)) . " " . ($prevstagebase+0) . " 0\n";
  print "" . (-($basevar+0)) . " " . (-($prevstagebase+1)) . " 0\n";
  #postcondition: PG
  print "" . (-($basevar+0)) . " " . ($nextstagebase+1) . " 0\n";

  #BSI
  #preconditions: KOP and PG and !SI
  print "" . (-($basevar+1)) . " " . ($prevstagebase+0) . " 0\n";
  print "" . (-($basevar+1)) . " " . ($prevstagebase+1) . " 0\n";
  print "" . (-($basevar+1)) . " " . (-($prevstagebase+2)) . " 0\n";
  #postcondition: SI
  print "" . (-($basevar+1)) . " " . ($nextstagebase+2) . " 0\n";

  #BSS
  #precondition: !SS
  print "" . (-($basevar+2)) . " " . (-($prevstagebase+3)) . " 0\n";
  #postcondition: SS
  print "" . (-($basevar+2)) . " " . ($nextstagebase+3) . " 0\n";

  #BPS
  #precondition: SS and SI and !PS
  print "" . (-($basevar+3)) . " " . ($prevstagebase+2) . " 0\n";
  print "" . (-($basevar+3)) . " " . ($prevstagebase+3) . " 0\n";
  print "" . (-($basevar+3)) . " " . (-($prevstagebase+4)) . " 0\n";
  #postcondition: PS
  print "" . (-($basevar+3)) . " " . ($nextstagebase+4) . " 0\n";

  #M...
  for $j (0..4) {
    #precondition: Nxx
    print "" . (-($basevar+4+2*$j)) . " " . (-($prevstagebase+$j)) . " 0\n";
    #postcondition: Nxx
    print "" . (-($basevar+4+2*$j)) . " " . (-($nextstagebase+$j)) . " 0\n";

    #precondition: xx
    print "" . (-($basevar+4+2*$j+1)) . " " . ($prevstagebase+$j) . " 0\n";
    #postcondition: xx
    print "" . (-($basevar+4+2*$j+1)) . " " . ($nextstagebase+$j) . " 0\n";
  }

  #Each literal requires an action to reach it

  #Any negation can be reached through MN___
  for $j (0..4) {
    print "" . ($nextstagebase+$j) . " " . ($basevar+4+2*$j) . " 0\n";
  }

  #KOP is always achieved with MKOP
  print "" . (-($nextstagebase+0)) . " " . ($basevar+5) . " 0\n";

  #Each other can be reached through M___ or B___
  for $j (1..4) {
    print "" . (-($nextstagebase+$j)) . " " . ($basevar+5+2*$j) . " " . ($basevar+$j-1) . " 0\n";
  }
}

