function [A,b,c,D,e] = load_max_flow(filename,sparse)
  if (nargin < 2 || sparse == 0)
    adjmat = csvread(filename);
    sparse = 0;
  else
    tmpadjmat = load(filename);
    adjmat = spconvert(tmpadjmat);
  end
  n = length(adjmat);
  nedges = nnz(adjmat);

  b = []; %always full
  if (sparse)
    c = spalloc(nedges,1,nnz(adjmat(:,n)));
    D = spalloc(n-2,nedges,2*nedges);
  else
    c = [];
    D = [];
  end

  nedges_sofar = 0;
  for src = 1:size(adjmat,1)
    for dst = 1:size(adjmat,2)
      weight = adjmat(src,dst);
      if (weight == 0)
        continue;
      end
      nedges_sofar = nedges_sofar + 1;
      if (src ~= 1 && src ~= n)
        D(src-1,nedges_sofar) = -1; %Flow is leaving a non-source/sink
      end
      if (dst ~= 1 && dst ~= n)
        D(dst-1,nedges_sofar) = 1; %Flow is entering a non-source/sink
      end
      if (src == n)
        c(nedges_sofar,1) = 1; %Flow is leaving the sink
      elseif (dst == n)
        c(nedges_sofar,1) = -1; %Flow is entering the sink
      end
      b(nedges_sofar,1) = weight;
    end
  end

  if (size(c,1) < nedges)
    c(nedges,1) = 0;
  end
  if (sparse)
    A = speye(nedges,nedges);
    e = spalloc(n-2,1,0);
  else
    A = eye(nedges,nedges);
    e = zeros(n-2,1);
  end
end

